/*
 * MokaBots - a package including bots to learn robocoding
 * Copyright (C) 2003  Simone Chiaretta
 * 
 * This file is part of Agenzia Viaggi.  
 *
 * MokaBots is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * For more information about this program please refer to the README file.
 *
 */


package moka;
import robocode.*;
import java.awt.Color;


/**
 * MokaEnhanced - a robot by Simone Chiaretta <simone@piyosailing.com>
 */
public class MokaEnhanced extends Robot
{
	// Costante PI greco
	static final double PI = Math.PI;
	
	// Variabili usate per salvare la posizione del nemico
	double enemyPosX;
	double enemyPosY;
	double enemyBearing;
	

		
	/**
	 * run: MokaEnhanced's default behavior
	 */
	public void run() {
		
		//Imposto il colore
		setColors(Color.red,Color.blue,Color.green);
		//Faccio una prima scansione del campo
		turnRadarRight(360);
		//Rendo indipendenti radar e torretta dalla base
		setAdjustGunForRobotTurn(true);
		setAdjustRadarForGunTurn(true);
		while(true) {
			ahead(200);
			turnRadarRight(360);
		}
	}

	/**
	 * onScannedRobot: What to do when you see another robot
	 */
	public void onScannedRobot(ScannedRobotEvent e) {
		//Calcolo la posizone del nemico
		calculatePosition(e.getDistance(),e.getBearingRadians());
		//Calcolo di quanto spostare la torretta per mirare al nemico
		double dest=Math.toDegrees(normaliseBearing(Math.toRadians(getGunHeading())-enemyBearing));
		//Stampo delle variabili per il debug
		out.println(Math.toDegrees(enemyBearing));
		out.println(getGunHeading());
		out.println(dest);
		//Giro torretta
		turnGunLeft(dest);
		//Saparo
		fire(2);
	}

	/**
	 * onHitByBullet: What to do when you're hit by a bullet
	 */
	public void onHitByBullet(HitByBulletEvent e) {
		turnRight(90-e.getBearing());
	}

	/**
	 * onHitByBullet: What to do when you're hit by a bullet
	 */
	public void onHitWall(HitWallEvent e) {
		turnRight(120 - e.getBearing());
	}

	/**
	 * onHitRobot: What to do when you hit a robot
	 */	
	public void onHitRobot(HitRobotEvent e) {
		back(100);
		turnRight(90 - e.getBearing());
	}
	
	/**
	 * Calcolo la posizone assoluta del robot
	 */
	public void calculatePosition(double distance, double bearing) {
		enemyBearing=bearing+Math.toRadians(getHeading());
	}

		
	/**
	 * Mi assicuro che l'angolo sia tra -180 e 180
	 */	
	public static double normaliseBearing(double ang) {
		if (ang > PI)
			ang -= 2*PI;
		if (ang < -PI)
			ang += 2*PI;
		return ang;
	}
	
}
